function [xbt,snr] = decwavt2sureletbit(r,x,jm,wavname,dec,sigmab,alpha,H,GGa,GGb,eta,rho,alseuili)
%[xbt,snr] = decwavt2sureletbit(r,x,jm,wavname,dec,sigmab,alpha,H,GGa,GGb,eta,rho,alseuili)
%
% INPUT:
% r: noisy image
% x: clean image
% jm: multiresolution decomposition level
% wavname : wavelet name
% sigmab : noise standard deviation
% alpha : coefficient needed in the exponential function considered
% in (Eq. 17)
% T. Blu, F. Luisier, "The SURE-LET Approach to Image Denoising," 
% IEEE Transactions on Image Processing, vol. 16, no. 11, pp. 2778-2786, 
% November 2007. 
% H: blur frequency response
% GGa: Wiener-like filter named G(p) (Eq. 45) in
% A SURE Approach for Digital Signal/Image Deconvolution Problems
% Jean-Christophe Pesquet, Amel Benazza-Benyahia, Caroline Chaux
% IEEE Transactions on Signal Processing, Vol. 57, No. 12, Dec. 2009, pp.
% 4616-4632. 
% GGb: identity operator in this case
% eta: lambda in (Eq. 45) (see above)
% rho and alseuili: parameters to be set for the computation of the
% variance of the estimate of the observable part of the quadratic risk
% They allowed us to compute the chi threshold (Eq. 31)
%
% OUTPUT:
% xbt: restored image
% snr: vector containing the snr snr(1) and psnr snr(2)
%
%
%*****************************************************
%* author: Jean-Christophe PESQUET                   *
%* institution: LIGM - UMR CNRS 8049         	     *
%*	       Université Paris-Est                      *
%* date: Friday, July, 24th 2009                     *
%*****************************************************

ind0 = find(H~=0);

seuila = min(abs(H(ind0)));
seuilb = max(abs(H(:)));

prec = (seuilb-seuila)/1000+eps;

if nargin == 12
    alseuili = 1/2;
end
seuili = (1-alseuili)*seuila+alseuili*seuilb;

mur = mean(r(:));
r = r-mur;

while 1
    fprintf('Threshold = %g : ',seuili);
    indp = find(abs(H) < seuili-eps);
    
    rs = projfour(r,indp);
    
    rc = real(ifft2(conj(GGa).*fft2(rs)));
    
    rt = projfour(r,indp,H);
    
    [wrc,sw] = wavedec2d(rc,jm,wavname,dec);
    
    [Gam,Vwb,Gamt,Kappa,Gamt2] =  calcgamt2(jm,sw,wavname,dec,indp,H,eta,GGa);
    
    [a,beta,fstein,v,vq,ETp] = decwavt2sureletbt(wrc,jm,sw,wavname,dec,sigmab,Gam,Vwb,Gamt,Kappa,alpha,indp,rt,GGb);
    
    [xbt,cstein,vstein] = est_LETt(a,beta,fstein,v,vq,ETp,Gamt2,sigmab);
    
    [E,E0,Et,Sb,Ss] = estcritt(xbt,rt,sigmab,H,indp,cstein,x-mur,vstein);
    
    fprintf(1,'Estimated criterion = %g true = %g troncated = %g optimized = %g to be optimized = %g\n',sum(E),sum(E0),Et,E(1),E0(1));
    E = sum(E);
    
    fprintf(1,'Noise variations = %g upper bound var = %g upper ratio = %g\n',Sb,Ss(2),E/Ss(2));
    
    xbt = xbt+mur;
    snr = calcsnr(xbt,x);
    message = sprintf('SNR = %g\t PSNR = %g',snr(1),snr(2));
    disp(message)
    
    if Ss(2) < E/rho
        seuilb = seuili;
    else
        seuila = seuili;
    end
    
    seuili_prec = seuili;
    seuili = (1-alseuili)*seuila+alseuili*seuilb;
    
    
    if abs(seuili_prec-seuili) <= prec
        break
    end
    
end